from simple_blogger.builder import PostBuilder
from simple_blogger.blogger import CachedBlogger
from simple_blogger.blogger.basic import SimpleBlogger, CommonBlogger
from simple_blogger.builder.task import TaskExtractor
from simple_blogger.cache.file_system import FileCache
from simple_blogger.builder.prompt import TaskPromptBuilder, ContentBuilderPromptBuilder
from simple_blogger.builder.content import CachedContentBuilder, ContentBuilder
import json, os

class CachedSimpleBlogger(SimpleBlogger, CachedBlogger):
    def __init__(self, posters, force_rebuild=False, index=None):
        CachedBlogger.__init__(self, force_rebuild=force_rebuild)
        SimpleBlogger.__init__(self, posters=posters, index=index)

    def _builder(self):
        tasks = json.load(open(self._tasks_file_path(), "rt", encoding="UTF-8")) if os.path.exists(self._tasks_file_path()) else []
        task_extractor = TaskExtractor(tasks=tasks, check=self._check_task)
        message_builder=CachedContentBuilder(
            task_builder=task_extractor,
            path_constructor=self._path_constructor,
            force_rebuild=self.force_rebuild,
            builder=ContentBuilder(
                generator=self._message_generator(), 
                prompt_builder=TaskPromptBuilder(
                        task_builder=task_extractor,
                        prompt_constructor=self._message_prompt_constructor
                    )
                ),
            cache=FileCache(root_folder=self._data_folder(), is_binary=False),
            filename=f"text"
        )
        media_builder=CachedContentBuilder(
            task_builder=task_extractor,
            path_constructor=self._path_constructor,
            force_rebuild=self.force_rebuild,
            builder=ContentBuilder(
                generator=self._image_generator(),
                prompt_builder=TaskPromptBuilder(
                        task_builder=task_extractor,
                        prompt_constructor=self._image_prompt_constructor
                    )
                ),
            cache=FileCache(root_folder=self._data_folder()),
            filename=f"image"
        )
        builder = PostBuilder(
            message_builder=message_builder,
            media_builder=media_builder
        )
        return builder
    
class CachedCommonBlogger(CommonBlogger, CachedBlogger):
    def __init__(self, posters, force_rebuild=False, index=None):
        CachedBlogger.__init__(self, force_rebuild=force_rebuild)
        CommonBlogger.__init__(self, posters=posters, index=index)
        
    def _builder(self):
        tasks = json.load(open(self._tasks_file_path(), "rt", encoding="UTF-8")) if os.path.exists(self._tasks_file_path()) else []
        task_extractor = TaskExtractor(tasks=tasks, check=self._check_task)
        message_builder=CachedContentBuilder(
            task_builder=task_extractor,
            path_constructor=self._path_constructor,
            builder=ContentBuilder(
                generator=self._message_generator(), 
                prompt_builder=TaskPromptBuilder(
                        task_builder=task_extractor,
                        prompt_constructor=self._message_prompt_constructor
                    )
            ),
            force_rebuild=self.force_rebuild,
            cache=FileCache(root_folder=self._data_folder(), is_binary=False),
            filename="text"
        )
        content_builder=CachedContentBuilder(
            task_builder=task_extractor,
            path_constructor=self._path_constructor,
            builder=ContentBuilder(
                generator=self._image_prompt_generator(), 
                prompt_builder=TaskPromptBuilder(
                    task_builder=task_extractor,
                    prompt_constructor=self._image_prompt_prompt_constructor
                )),
            force_rebuild=self.force_rebuild,
            filename="image_prompt",
            cache=FileCache(root_folder=self._data_folder(), is_binary=False)
        )
        media_builder=CachedContentBuilder(
            task_builder=task_extractor,
            path_constructor=self._path_constructor,
            builder=ContentBuilder(
                generator=self._image_generator(),
                prompt_builder=ContentBuilderPromptBuilder(
                    content_builder=content_builder
                )
            ),
            force_rebuild=self.force_rebuild,
            cache=FileCache(root_folder=self._data_folder()),
            filename="image"
        )
        builder = PostBuilder(
            message_builder=message_builder,
            media_builder=media_builder
        )
        return builder
    
    

    


from simple_blogger.blogger.basic.cached import CachedSimpleBlogger, CachedCommonBlogger
from simple_blogger.blogger.auto import AutoBlogger

class CachedAutoSimpleBlogger(CachedSimpleBlogger, AutoBlogger):
    def __init__(self, posters, first_post_date=None, force_rebuild=False, index=None):
        AutoBlogger.__init__(self, first_post_date)
        CachedSimpleBlogger.__init__(self, posters, force_rebuild, index)

class CachedAutoCommonBlogger(CachedCommonBlogger, AutoBlogger):
    def __init__(self, posters, first_post_date=None, force_rebuild=False, index=None):
        AutoBlogger.__init__(self, first_post_date)
        CachedCommonBlogger.__init__(self, posters, force_rebuild, index)   
    


from simple_blogger.uploader.S3Uploader import S3Uploader
from simple_blogger.preprocessor.text import SerialProcessor, MarkdownCleaner, IdentityProcessor
from simple_blogger.poster import IPoster, Post
import os, requests

class TikTokVideoPoster(IPoster):
    BASE_URL = 'https://open.tiktokapis.com/v2'

    def __init__(self, user_token_name='TT_BOT_TOKEN', review_mode=True, uploader=None, processor=None):
        self.uploader = uploader or S3Uploader()
        self.token = os.environ.get(user_token_name)
        self.review_mode = review_mode
        self.processor = SerialProcessor([MarkdownCleaner(), processor or IdentityProcessor()])
            
    def check_status(self, publish_id, user_token_name=None):
        token = (user_token_name and os.environ.get(user_token_name)) or self.token
        fetch_url = f'{TikTokVideoPoster.BASE_URL}/post/publish/status/fetch/'
        headers = {
            'Authorization': f"Bearer {token}",
            'Content-Type': 'application/json; charset=UTF-8'
        }
        body = {
            "publish_id": f"{publish_id}"
        }
        response = requests.post(url=fetch_url, json=body, headers=headers).json()
        return response
    
    def cancel(self, publish_id, user_token_name=None):
        token = (user_token_name and os.environ.get(user_token_name)) or self.token
        fetch_url = f'{TikTokVideoPoster.BASE_URL}/post/publish/cancel/ '
        headers = {
            'Authorization': f"Bearer {token}",
            'Content-Type': 'application/json; charset=UTF-8'
        }
        body = {
            "publish_id": f"{publish_id}"
        }
        response = requests.post(url=fetch_url, json=body, headers=headers).json()
        return response
    
    def list_videos(self, user_token_name=None):
        token = (user_token_name and os.environ.get(user_token_name)) or self.token
        fetch_url = f'{TikTokVideoPoster.BASE_URL}/video/list/?fields=cover_image_url,id,title'
        headers = {
            'Authorization': f"Bearer {token}",
            'Content-Type': 'application/json'
        }
        body = {
            'max_count': 20
        }
        response = requests.post(url=fetch_url, json=body, headers=headers).json()
        return response
    
    def _review_url(self, post:Post, user_token_name):
        token = user_token_name
        url = self.uploader.upload(post.media)
        init_url = f'{TikTokVideoPoster.BASE_URL}/post/publish/inbox/video/init/'
        headers = {
            'Authorization': f"Bearer {token}",
            'Content-Type': 'application/json'
        }
        body = {
            'source_info': {
                'source': 'PULL_FROM_URL',
                "video_url": f"{url}"
            }
        }
        response = requests.post(url=init_url, json=body, headers=headers).json()
        publish_id = response['data']['publish_id']
        return publish_id

    def _review(self, post:Post, user_token_name):
        token = user_token_name
        init_url = f'{TikTokVideoPoster.BASE_URL}/post/publish/inbox/video/init/'
        headers = {
            'Authorization': f"Bearer {token}",
            'Content-Type': 'application/json'
        }
        video_size = len(post.get_real_media().read())
        body = {
            'source_info': {
                'source': 'FILE_UPLOAD',
                'video_size': video_size,
                'chunk_size': video_size,
                'total_chunk_count': 1
            }
        }
        response = requests.post(url=init_url, json=body, headers=headers).json()
        upload_url = response['data']['upload_url']
        publish_id = response['data']['publish_id']
        headers = {
            'Content-Type': post.media.get_content_type(),
            'Content-Range' : f"bytes 0-{video_size-1}/{video_size}",
            'Content-Length': f"{video_size}",
        }
        response = requests.put(url=upload_url, headers=headers, data=post.get_real_media())
        return publish_id

    def _send(self, post:Post, user_token_name, processor):
        token = user_token_name
        init_url = f'{TikTokVideoPoster.BASE_URL}/post/publish/video/init/'
        headers = {
            'Authorization': f"Bearer {token}",
            'Content-Type': 'application/json; charset=UTF-8'
        }
        video_size = len(post.get_real_media().read())
        title = post.get_real_message(processor)
        body = {
            'post_info' : {
                'privacy_level': 'PUBLIC_TO_EVERYONE',
                'title': title, 
                'is_aigc': True,
            }, 
            'source_info': {
                'source': 'FILE_UPLOAD',
                'video_size': video_size,
                'chunk_size': video_size,
                'total_chunk_count': 1
            }
        }
        response = requests.post(url=init_url, json=body, headers=headers).json()
        upload_url = response['data']['upload_url']
        publish_id = response['data']['publish_id']
        headers = {
            'Content-Type': post.media.get_content_type(),
            'Content-Range' : f"bytes 0-{video_size-1}/{video_size}",
            'Content-Length': f"{video_size}",
        }
        response = requests.put(url=upload_url, headers=headers, data=post.get_real_media())
        return publish_id

    def post(self, post:Post, user_token_name=None, processor=None):
        token = (user_token_name and os.environ.get(user_token_name)) or self.token
        processor = SerialProcessor([self.processor, processor or IdentityProcessor()])
        if self.review_mode:
            return self._review(post, user_token_name=token)
        else:
            self._send(post, user_token_name=token, processor=processor) 


import os, boto3, uuid
from simple_blogger.generator import File

class S3Uploader():
    def __init__(self, key_id=None, secret=None, bucket=None, endpoint='https://storage.yandexcloud.net'):
        key_id = key_id or os.environ.get('S3_KEY_ID')
        secret = secret or os.environ.get('S3_SECRET')
        self.bucket = bucket or os.environ.get('S3_BUCKET')
        self.client = boto3.client("s3"
                            , aws_access_key_id=key_id
                            , aws_secret_access_key=secret
                            , endpoint_url=endpoint)

    def upload(self, file:File):
        file_name = str(uuid.uuid4())
        self.client.upload_fileobj(file.get_file(), self.bucket, file_name, ExtraArgs={ 'ContentType': file.get_content_type() })
        return self.client.generate_presigned_url(ClientMethod='get_object', Params={ 'Bucket': self.bucket, 'Key': file_name })


from markdown import Markdown
from io import StringIO

def unmark_element(element, stream=None):
    if stream is None:
        stream = StringIO()
    if element.text:
        stream.write(element.text)
    for sub in element:
        unmark_element(sub, stream)
    if element.tail:
        stream.write(element.tail)
    return stream.getvalue()

Markdown.output_formats["plain"] = unmark_element


from abc import ABC, abstractmethod
from markdown import Markdown
import emoji, re

class ITextProcessor(ABC):
    @abstractmethod
    def process(self, message:str)->str:
        """ Message preprocess method """

class IdentityProcessor(ITextProcessor):        
    def process(self, message:str)->str:
        return message
    
class MarkdownCleaner():
    def __init__(self):
        self.md = Markdown(output_format="plain")
        self.md.stripTopLevelTags = False

    def process(self, message:str)->str:
        return self.md.convert(message)
    
class EmojiCleaner(ITextProcessor):        
    def process(self, message:str)->str:
        return emoji.replace_emoji(message)
    
class SerialProcessor(ITextProcessor):
    def __init__(self, processors:list[ITextProcessor]):
        self.processors = processors

    def process(self, message:str)->str:
        for processor in self.processors:
            message = processor.process(message=message)
        return message
    
class TagAdder(ITextProcessor):
    def __init__(self, tags:list[str]):
        self.tags = tags

    def process(self, message:str)->str:
        text_lower = message.lower()
        delimiter = '\n\n'
        for tag in self.tags:
            if not tag.lower() in text_lower:
                message += f"{delimiter}{tag}"
                delimiter = ' '
        return message
    
class OkCleaner(ITextProcessor):        
    def process(self, message:str)->str:
        if re.match(r'\Aконечно|ок', message, re.IGNORECASE):
            message = re.sub(r'\A[^.!&]+.', '', message)
        return re.sub(r'\A\s+', '', message)


from simple_blogger.builder import PostBuilder
from simple_blogger.blogger import CachedBlogger
from simple_blogger.blogger.shorts import ShortsBlogger
from simple_blogger.generator.video.basic import ImageSequenceGenerator
from simple_blogger.builder.shorts import ShortsBuilder
from simple_blogger.generator.subs import SubsGenerator
from simple_blogger.builder.task import TaskExtractor
from simple_blogger.cache.file_system import FileCache
from simple_blogger.builder.prompt import TaskPromptBuilder, ContentBuilderPromptBuilder
from simple_blogger.builder.content import CachedContentBuilder, ContentBuilder
from simple_blogger.builder.tools import DurationBuilder
from simple_blogger.preprocessor.text import MarkdownCleaner, SerialProcessor, EmojiCleaner
import json

class CachedShortsBlogger(ShortsBlogger, CachedBlogger):
    def __init__(self, posters, image_count=5, force_rebuild=False, index=None):
        ShortsBlogger.__init__(self, posters, image_count, index)
        CachedBlogger.__init__(self, force_rebuild)
        
    def _builder(self):
        tasks = json.load(open(self._tasks_file_path(), "rt", encoding="UTF-8"))
        task_extractor = TaskExtractor(tasks=tasks, check=self._check_task)
        message_builder = CachedContentBuilder(
                            task_builder=task_extractor,
                            path_constructor=self._path_constructor,
                            builder=ContentBuilder(
                                generator=self._message_generator(), 
                                prompt_builder=TaskPromptBuilder(
                                        task_builder=task_extractor,
                                        prompt_constructor=self._message_prompt_constructor
                                    )
                            ),
                            force_rebuild=self.force_rebuild,
                            cache=FileCache(root_folder=self._data_folder(), is_binary=False),
                            filename="text"
                        )
        audio_builder=CachedContentBuilder(
                            task_builder=task_extractor,
                            path_constructor=self._path_constructor,
                            builder=ContentBuilder(
                                generator=self._speech_generator(), 
                                prompt_builder=ContentBuilderPromptBuilder(content_builder=message_builder),
                                prompt_processor=SerialProcessor([MarkdownCleaner(), EmojiCleaner()])
                            ),
                            force_rebuild=self.force_rebuild,
                            filename="audio",
                            cache=FileCache(root_folder=self._data_folder())
                        )
        duration_builder=DurationBuilder(audio_builder)
        subs_builder=CachedContentBuilder(
                        task_builder=task_extractor,
                        path_constructor=self._path_constructor,
                        builder=ContentBuilder(
                            generator=SubsGenerator(duration_constructor=duration_builder.build), 
                            prompt_builder=ContentBuilderPromptBuilder(content_builder=message_builder),
                            prompt_processor=SerialProcessor([MarkdownCleaner(), EmojiCleaner()])
                        ),
                        force_rebuild=self.force_rebuild,
                        filename="subs",
                        cache=FileCache(root_folder=self._data_folder(), is_binary=False)                            
                    )        
        image_prompt_builder=ContentBuilderPromptBuilder(
                                content_builder=CachedContentBuilder(
                                    task_builder=task_extractor,
                                    path_constructor=self.__path_constructor,
                                    builder=ContentBuilder(
                                        generator=self._image_prompt_generator(), 
                                        prompt_builder=TaskPromptBuilder(
                                            task_builder=task_extractor,
                                            prompt_constructor=self._image_prompt_prompt_constructor
                                        )),
                                    force_rebuild=self.force_rebuild,
                                    filename="image_prompt",
                                    cache=FileCache(root_folder=self._data_folder(), is_binary=False)
                                )
                            )
        video_builder = CachedContentBuilder(
                            task_builder=task_extractor,
                            path_constructor=self._path_constructor,
                            builder=ContentBuilder(
                                generator=ImageSequenceGenerator(self._image_generator(), duration_builder.build, self.image_count),
                                prompt_builder=image_prompt_builder,
                            ),
                            force_rebuild=self.force_rebuild,
                            cache=FileCache(root_folder=self._data_folder()),
                            filename="video"
                        )
        media_builder = CachedContentBuilder(
                            task_builder=task_extractor,
                            path_constructor=self._path_constructor,
                            builder=ShortsBuilder(audio_builder, subs_builder, video_builder),
                            force_rebuild=self.force_rebuild,
                            cache=FileCache(root_folder=self._data_folder()),
                            filename="media"
                        )

        builder = PostBuilder(
            message_builder=message_builder,
            media_builder=media_builder
        )
        return builder


from dataclasses import dataclass
from abc import ABC, abstractmethod
from io import IOBase
from PIL import Image
from io import BytesIO, StringIO

@dataclass
class File:
    ext: str
    file: IOBase

    def get_file(self):
        self.file.seek(0)
        return self.file
    
    ext2ct = {
        'jpeg': 'image/jpeg',
        'jpg' : 'image/jpeg',
        'png' : 'image/png',
        'txt' : 'text/plain',
        'mp4' : 'video/mp4'
    }

    def get_content_type(self)->str:
        if self.ext in File.ext2ct:
            return File.ext2ct[self.ext]
        return 'application/octet-stream'
    
class IGeneratorBase(ABC):    
    @abstractmethod
    def generate(self, prompt)->File:
        """Generation method"""

    @abstractmethod
    def ext()->str:
        """Content file extension"""
    
class TextGenerator(IGeneratorBase):
    def __init__(self, system_prompt):
        self.system_prompt = system_prompt

    def generate(self, prompt, **_):
        return File(self.ext(), StringIO(prompt))
    
    def ext(self):
        return 'txt'

class ImageGenerator(IGeneratorBase):
    def __init__(self):
        pass

    def generate(self, *_, **__):
        stream = BytesIO()
        Image.new(mode='RGBA', size=(320, 240), color=1).save(stream, format=self.ext())
        return File(self.ext(), stream)
    
    def ext(self):
        return 'png'


from simple_blogger.generator import File, TextGenerator, ImageGenerator
from io import StringIO, BytesIO
from yandex_cloud_ml_sdk import YCloudML
import os

class YandexTextGenerator(TextGenerator):
    def __init__(self, system_prompt, folder_id=None, model_name='yandexgpt', model_version='latest', creativity=0.5):
        super().__init__(system_prompt=system_prompt)
        self.folder_id=folder_id or os.environ.get('YC_FOLDER_ID')
        self.model_name=model_name
        self.model_version=model_version
        self.creativity=creativity

    def generate(self, prompt, **_):
        sdk = YCloudML(folder_id=self.folder_id)
        model = sdk.models.completions(model_name=self.model_name, model_version=self.model_version)
        model.configure(temperature=self.creativity)
        text = model.run([
                        { "role": "system", "text": self.system_prompt },
                        { "role": "user", "text": prompt },
                    ]
                ).alternatives[0].text
        return File(self.ext(), StringIO(text))
    
class YandexImageGenerator(ImageGenerator):
    def __init__(self, folder_id=None, model_name='yandex-art', model_version='latest', system_prompt=None, style_prompt=None):
        super().__init__()
        self.folder_id=folder_id or os.environ.get('YC_FOLDER_ID')
        self.model_name=model_name
        self.model_version=model_version
        self.system_prompt=system_prompt
        self.style_prompt=style_prompt

    def generate(self, prompt, **_):
        sdk = YCloudML(folder_id=self.folder_id)
        model = sdk.models.image_generation(self.model_name)
        prompt = f"{self.system_prompt + '\n' if self.system_prompt else ''}{prompt}{'\n' + self.style_prompt if self.style_prompt else ''}"
        operation = model.run_deferred(prompt)
        result = operation.wait()
        return File(self.ext(), BytesIO(result.image_bytes))
    
    def ext(self):
        return 'jpeg'


import os, requests, vk
from simple_blogger.generator import File

class VkUploader():
    def __init__(self, token_name='VK_BOT_TOKEN', group_id=None):
        token = os.environ.get(token_name)
        self.group_id = group_id or os.environ.get('VK_REVIEW_GROUP_ID')
        self.api = vk.API(token, v='5.199')

    def upload_photo(self, file:File, group_id=None):
        group_id = group_id or self.group_id
        response = self.api.photos.getWallUploadServer(group_id=group_id)
        upload_url = response["upload_url"]
        files = {'photo': (f"photo.{file.ext}", file.get_file(), file.get_content_type() ) }
        for i in range(10):
            response = requests.post(url=upload_url, files=files).json()
            if response["photo"] != '': 
                break
        response = self.api.photos.saveWallPhoto(group_id=self.group_id
                                                , photo=response["photo"]
                                                , server=response["server"]
                                                , hash=response["hash"])
        return f"photo{response[0]['owner_id']}_{response[0]['id']}"
    
    def upload_video(self, file:File, group_id=None):
        group_id = group_id or self.group_id
        response = self.api.video.save(group_id=group_id)
        upload_url = response["upload_url"]
        video_id = response['video_id']
        owner_id = response['owner_id']
        files = {'video_file': (f"video.{file.ext}", file.get_file(), file.get_content_type() ) }
        response = requests.post(url=upload_url, files=files).json()
        return f"video{owner_id}_{video_id}"
    
    def upload_photo_for_stories(self, file:File, group_id=None):
        group_id = group_id or self.group_id
        response = self.api.stories.getPhotoUploadServer(group_id=group_id, add_to_news=1)
        upload_url = response["upload_url"]
        files = {'file': (f"photo.{file.ext}", file.get_file(), file.get_content_type() ) }
        response = requests.post(url=upload_url, files=files).json()
        return response['response']['upload_result']

    def upload_video_for_stories(self, file:File, group_id=None):
        group_id = group_id or self.group_id
        response = self.api.stories.getVideoUploadServer(group_id=group_id, add_to_news=1)
        upload_url = response["upload_url"]
        files = {'video_file': (f"video.{file.ext}", file.get_file(), file.get_content_type() ) }
        response = requests.post(url=upload_url, files=files).json()
        return response['response']['upload_result']



from simple_blogger.blogger.basic import SimpleBlogger, CommonBlogger, ProjectBlogger
from datetime import date, timedelta

class AutoBlogger(ProjectBlogger):
    def __init__(self, first_post_date=None):
        self.first_post_date = first_post_date or date.today()

    def _check_task(self, task, tasks, days_before=0):
        check_date = date.today() + timedelta(days=days_before)
        days_diff = check_date - self.first_post_date
        return task["day"] == days_diff.days % len(tasks)
    
    def create_auto_tasks(self, day_offset=0, days_between_posts=1, multiple_projects=False, shuffle=True):
        project_tasks=self._load_project_tasks(multiple_projects)
        if shuffle: self._shuffle(project_tasks)
        self.__set_days(project_tasks, day_offset, days_between_posts)
        self._save_tasks(project_tasks=project_tasks)

    def __set_days(self, project_tasks, day_offset=0, days_between_posts=1):
        for tasks in project_tasks:
            day=day_offset
            for task in tasks:
                task["day"] = day
                day += days_between_posts

class AutoSimpleBlogger(SimpleBlogger, AutoBlogger):
    def __init__(self, posters, first_post_date=None, index=None):
        AutoBlogger.__init__(self, first_post_date)
        SimpleBlogger.__init__(self, posters, index)

class AutoCommonBlogger(CommonBlogger, AutoBlogger):
    def __init__(self, posters, first_post_date=None, index=None):
        AutoBlogger.__init__(self, first_post_date)
        CommonBlogger.__init__(self, posters, index)


from simple_blogger.blogger.basic import SimpleBlogger, CommonBlogger, ProjectBlogger
from datetime import date, timedelta, datetime
import math

class FiniteBlogger(ProjectBlogger):
    def _set_dates(self, project_tasks, first_post_date=None, days_between_posts=1):
        first_post_date = first_post_date or date.today()
        days_between_posts = timedelta(days=days_between_posts)
        for tasks in project_tasks:
            curr_date = first_post_date
            for task in tasks:
                task["date"] = curr_date.strftime("%Y-%m-%d")
                curr_date += days_between_posts

    def create_simple_tasks(self, first_post_date=None, days_between_posts=1, multiple_projects=False, shuffle=True):
        project_tasks=self._load_project_tasks(multiple_projects)
        if shuffle: self._shuffle(project_tasks)
        self.__set_dates(project_tasks, first_post_date, days_between_posts)
        self._save_tasks(project_tasks)

    def __set_dates(self, project_tasks, first_post_date=None, days_between_posts=1):
        first_post_date = first_post_date or date.today()
        days_between_posts = timedelta(days=days_between_posts)
        for tasks in project_tasks:
            curr_date = first_post_date
            for task in tasks:
                task["date"] = curr_date.strftime("%Y-%m-%d")
                curr_date += days_between_posts

    def create_tasks_between(self, first_post_date, last_post_date, exclude_weekends=True, multiple_projects=False, shuffle=True):
        project_tasks=self._load_project_tasks(multiple_projects)
        if shuffle: self._shuffle(project_tasks)
        self.__set_dates_between(project_tasks, first_post_date, last_post_date, exclude_weekends)
        self._save_tasks(project_tasks=project_tasks)

    def __set_dates_between(self, project_tasks, first_post_date, last_post_date, exclude_weekends):
        first_post_date = datetime.fromordinal(first_post_date.toordinal())
        last_post_date = datetime.fromordinal(last_post_date.toordinal())
        for i, tasks in enumerate(project_tasks):
            curr_date = first_post_date + timedelta(days=math.trunc(i/2))
            d = (last_post_date - curr_date).days / len(tasks)
            days_between_posts = timedelta(days = d)
            for task in tasks:
                if exclude_weekends:
                    if curr_date.weekday() == 6: curr_date += timedelta(days=1)
                    if curr_date.weekday() == 5: curr_date += timedelta(days=-1)
                task["date"] = curr_date.strftime("%Y-%m-%d")
                curr_date += days_between_posts

class FiniteSimpleBlogger(SimpleBlogger, FiniteBlogger):
    def __init__(self, posters, index=None):
        SimpleBlogger.__init__(self, posters, index)

class FiniteCommonBlogger(CommonBlogger, FiniteBlogger):
    def __init__(self, posters, index=None):
        CommonBlogger.__init__(self, posters, index)


from __future__ import annotations 
import simple_blogger.builder.content

from abc import ABC, abstractmethod

class IPromptBuilder(ABC):
    @abstractmethod
    def build(self, *_, **__)->str:
        """ Prompt builder method """

class IdentityPromptBuilder(IPromptBuilder):
    def __init__(self, prompt:str):
        self.prompt = prompt
    
    def build(self, *_, **__):
        return self.prompt
    
    def ext(self):
        return 'txt'
    
class TaskPromptBuilder(IPromptBuilder):
    def __init__(self, task_builder, prompt_constructor):
       self.task_builder=task_builder
       self.prompt_constructor=prompt_constructor

    def build(self, *_, **__):
        task = self.task_builder.build()
        return task and self.prompt_constructor(task)
    
    def ext(self):
        return self.task_builder.ext()

class ContentBuilderPromptBuilder(IPromptBuilder):
    def __init__(self, content_builder: simple_blogger.builder.content.IContentBuilder):
        self.content_builder=content_builder
    
    def build(self):
        return (content:=self.content_builder.build()) and content.get_file().read()
    
    def ext(self):
        return self.content_builder.ext()


from simple_blogger.blogger.basic.cached import CachedSimpleBlogger, CachedCommonBlogger
from simple_blogger.blogger.finite import FiniteBlogger

class CachedFiniteSimpleBlogger(CachedSimpleBlogger, FiniteBlogger):
    def __init__(self, posters, force_rebuild=False, index=None):
        CachedSimpleBlogger.__init__(self, posters, force_rebuild, index)

class CachedFiniteCommonBlogger(CachedCommonBlogger, FiniteBlogger):
    def __init__(self, posters, force_rebuild=False, index=None):
        CachedCommonBlogger.__init__(self, posters, force_rebuild, index)   
    


from simple_blogger.uploader.VkUploader import VkUploader
from simple_blogger.preprocessor.text import SerialProcessor, MarkdownCleaner, IdentityProcessor
from simple_blogger.poster import IPoster, Post
import os, vk

class VkPoster(IPoster):
    def __init__(self, token_name='VK_BOT_TOKEN', group_id=None, uploader=None, processor=None, **_):
        token = os.environ.get(token_name)
        self.group_id = group_id or os.environ.get('VK_REVIEW_GROUP_ID')
        self.api = vk.API(token, v='5.199')
        self.uploader = uploader or VkUploader(group_id=self.group_id)
        self.processor = SerialProcessor([MarkdownCleaner(), processor or IdentityProcessor()])
            
    def _upload(self, file, group_id):
        return self.uploader.upload_photo(file=file, group_id=group_id)
    
    def post(self, post:Post, group_id=None, processor=None, **_):
        processor = processor or IdentityProcessor()
        group_id = group_id or self.group_id
        if post.media and post.message:
            image_address = self._upload(post.media, group_id)
            caption = post.get_real_message(SerialProcessor([self.processor, processor]))
            self.api.wall.post(owner_id=f"-{group_id}", from_group=1, message=caption, attachments=f"{image_address}")
        else:
            if post.media:
                image_address = self._upload(post.media, group_id)
                self.api.wall.post(owner_id=f"-{group_id}", from_group=1, attachments=f"{image_address}")
            if post.message:
                caption = post.get_real_message(SerialProcessor([self.processor, processor]))
                self.api.wall.post(owner_id=f"-{group_id}", from_group=1, message=caption)

class VkVideoPoster(VkPoster):
    def _upload(self, file, group_id):
        return self.uploader.upload_video(file=file, group_id=group_id)

class VkVideoStoriesPoster(VkPoster):
    def post(self, post:Post, group_id=None, processor=None, **_):
        processor = processor or IdentityProcessor()
        group_id = group_id or self.group_id
        if post.media:
            upload_result = self.uploader.upload_video_for_stories(file=post.media, group_id=group_id)
            result = self.api.stories.save(upload_results=upload_result)
            return result



import os

class FileCache():
    def __init__(self, root_folder='.', is_binary=True):
        self.root_folder = root_folder
        self.is_binary = is_binary

    def load(self, uri):
        path = f"{self.root_folder}/{uri}"
        if os.path.exists(path = path):
            return open(file = path, 
                        mode = 'rb' if self.is_binary else 'rt',
                        encoding = None if self.is_binary else 'UTF-8') 
        else: 
            return None
        
    def save(self, uri, io_base):
        path = f"{self.root_folder}/{uri}"
        folder = os.path.dirname(path)
        os.makedirs(folder, exist_ok=True)
        open(file = path, 
             mode = 'wb' if self.is_binary else 'wt',
             encoding = None if self.is_binary else 'UTF-8').write(io_base.getvalue())


from simple_blogger.generator import File
from simple_blogger.preprocessor.text import IdentityProcessor
from abc import ABC, abstractmethod
from dataclasses import dataclass
from io import IOBase

@dataclass
class Post:
    message: File
    media: File

    def get_real_message(self, processor=IdentityProcessor())->str:
        if self.message:
            return processor.process(self.message.get_file().read())
        return None
    
    def get_real_media(self)->IOBase:
        if self.media:
            return self.media.get_file()
        return None
        
class IPoster(ABC):
    @abstractmethod
    def post(self, post:Post, **_):
        """ Post method """



from simple_blogger.builder import PostBuilder
from simple_blogger.blogger import ProjectBlogger
from simple_blogger.generator.yandex import YandexTextGenerator
from simple_blogger.builder.prompt import TaskPromptBuilder, ContentBuilderPromptBuilder
from simple_blogger.builder.content import ContentBuilder
    
class SimpleBlogger(ProjectBlogger):
    def _image_prompt_constructor(self, task):
        return f"Нарисуй рисунок, вдохновленный темой {task['topic']} из области '{task['category']}'"
    
    def _builder(self):
        tasks = self._load_tasks()
        task_extractor = self._task_extractor(tasks)
        message_builder=ContentBuilder(
            generator=self._message_generator(), 
            prompt_builder=TaskPromptBuilder(
                task_builder=task_extractor,
                prompt_constructor=self._message_prompt_constructor
            )
        )
        media_builder=ContentBuilder(
            generator=self._image_generator(),
            prompt_builder=TaskPromptBuilder(
                task_builder=task_extractor,
                prompt_constructor=self._image_prompt_constructor
            )
        )
        builder = PostBuilder(
            message_builder=message_builder,
            media_builder=media_builder
        )
        return builder
    
    def print_current_task(self):
        task = super().print_current_task()
        print(f"image: {self._image_prompt_constructor(task)}")
        return task 
    
class CommonBlogger(ProjectBlogger):        
    def _image_prompt_prompt_constructor(self, task):
        return f"Напиши промпт для генерации изображения на тему '{task['topic']}' из области '{task['category']}'"
    
    def _image_prompt_generator(self):
        return YandexTextGenerator(system_prompt=self._system_prompt())
    
    def print_current_task(self):
        task = super().print_current_task()
        print(f"image_prompt: {self._image_prompt_prompt_constructor(task)}")
        return task
    
    def _builder(self):
        tasks = self._load_tasks()
        task_extractor = self._task_extractor(tasks)
        message_builder=ContentBuilder(
            generator=self._message_generator(), 
            prompt_builder=TaskPromptBuilder(
                task_builder=task_extractor,
                prompt_constructor=self._message_prompt_constructor
            )
        )
        image_prompt_builder=ContentBuilder(
            generator=self._image_prompt_generator(), 
            prompt_builder=TaskPromptBuilder(
                task_builder=task_extractor,
                prompt_constructor=self._image_prompt_prompt_constructor
            )
        )
        media_builder=ContentBuilder(
            generator=self._image_generator(),
            prompt_builder=ContentBuilderPromptBuilder(
                content_builder=image_prompt_builder
            )
        )
        builder = PostBuilder(
            message_builder=message_builder,
            media_builder=media_builder
        )
        return builder


import os, telebot
from simple_blogger.poster import Post, IPoster
from simple_blogger.preprocessor.text import SerialProcessor, IdentityProcessor

class TelegramPoster(IPoster):
    def __init__(self, bot_token_name='TG_BOT_TOKEN', chat_id=None, processor=None, send_text_with_image=True, **_):
        self.bot_token = os.environ.get(bot_token_name)
        self.chat_id = chat_id or os.environ.get('TG_REVIEW_CHAT_ID')
        self.send_text_with_image = send_text_with_image
        self.bot = telebot.TeleBot(self.bot_token)
        self.processor = processor or IdentityProcessor()

    def post(self, post:Post, chat_id=None, processor=None, **_):
        processor = processor or IdentityProcessor()
        chat_id = chat_id or self.chat_id
        if self.send_text_with_image and post.media and post.message:
            self.bot.send_photo(chat_id=chat_id, photo=post.get_real_media(), caption=post.get_real_message(SerialProcessor([self.processor, processor])), parse_mode="Markdown")
        else:
            if post.media:
                self.bot.send_photo(chat_id=chat_id, photo=post.get_real_media(), disable_notification=True)
            if post.message:
                self.bot.send_message(chat_id=chat_id, text=post.get_real_message(SerialProcessor([self.processor, processor])), parse_mode="Markdown")
    
    def post_error(self, message):
        self.bot.send_message(chat_id=self.chat_id, text=message)

class TelegramVideoPoster(TelegramPoster):
    def post(self, post:Post, chat_id=None, processor=None, **_):
        processor = processor or IdentityProcessor()
        chat_id = chat_id or self.chat_id
        if self.send_text_with_image and post.media and post.message:
            self.bot.send_video(chat_id=chat_id, video=post.get_real_media(), caption=post.get_real_message(SerialProcessor([self.processor, processor])), parse_mode="Markdown")
        else:
            if post.media:
                self.bot.send_video(chat_id=chat_id, video=post.get_real_media(), disable_notification=True)
            if post.message:
                self.bot.send_message(chat_id=chat_id, text=post.get_real_message(SerialProcessor([self.processor, processor])), parse_mode="Markdown")



from simple_blogger.blogger.shorts import ShortsBlogger
from simple_blogger.blogger.shorts.cached import CachedShortsBlogger
from simple_blogger.blogger.auto import AutoBlogger

class AutoCommonShortsBlogger(ShortsBlogger, AutoBlogger):
    def __init__(self, posters, first_post_date=None, image_count=5, index=None):
        ShortsBlogger.__init__(self, posters, image_count, index)
        AutoBlogger.__init__(self,first_post_date)

class CachedAutoCommonShortsBlogger(CachedShortsBlogger, AutoBlogger):
    def __init__(self, posters, first_post_date=None, image_count=5, force_rebuild=False, index=None):
        CachedShortsBlogger.__init__(self, posters, image_count, force_rebuild, index)
        AutoBlogger.__init__(self,first_post_date)


from simple_blogger.generator import File, IGeneratorBase
from pydub import AudioSegment
from abc import abstractmethod
from io import BytesIO

class ISpeechGeneratorBase(IGeneratorBase):    
    @abstractmethod
    def generate(self, text_to_speak)->File:
        """Generation method"""

class SpeechGenerator(ISpeechGeneratorBase):
    def __init__(self):
        pass

    def generate(self, _):       
        stream = BytesIO()
        AudioSegment.empty().export(stream)
        return File(self.ext(), stream)
    
    def ext(self):
        return 'mp3'


from simple_blogger.builder import PostBuilder
from simple_blogger.builder.content import CachedContentBuilder, ContentBuilder
from simple_blogger.builder.prompt import ContentBuilderPromptBuilder, TaskPromptBuilder
from simple_blogger.cache import Cache
from simple_blogger.poster import IPoster
from simple_blogger.generator.yandex import YandexTextGenerator, YandexImageGenerator
from simple_blogger.builder.task import IdentityTaskBuilder, TaskExtractor
from abc import abstractmethod
from datetime import date, timedelta
import json, os, random

from simple_blogger.preprocessor.text import EmojiCleaner, MarkdownCleaner, SerialProcessor

class SimplestBlogger():
    def __init__(self, builder:PostBuilder, posters:list[IPoster]):
        self.builder = builder
        self.posters = posters

    def post(self, **__):
        post = self.builder.build()
        for poster in self.posters:
            poster.post(post=post)
    
    def _system_prompt(self):
        return 'Ты - известный блоггер с 1000000 подписчиков'

    def _message_generator(self):
        return YandexTextGenerator(system_prompt=self._system_prompt())
    
    def _image_generator(self):
        return YandexImageGenerator()
    
class Journalist(SimplestBlogger):
    def _prompt_constructor(self, *_, **__):
        return "Выбери рандомную тему из рандомной области. Напиши эссе на эту тему. Используй смайлики и не более 150 слов."

    def __init__(self, posters):
        task_builder = IdentityTaskBuilder('.')
        message_builder=CachedContentBuilder(
            task_builder=task_builder,
            path_constructor=lambda _:'.',
            builder=ContentBuilder(
                generator=self._message_generator(),
                prompt_builder=TaskPromptBuilder(
                        task_builder=task_builder,
                        prompt_constructor=self._prompt_constructor
                    )
            ),
            cache=Cache(),
            filename=f"text"
        )
        image_builder=ContentBuilder(
            generator=self._image_generator(), 
            prompt_builder=ContentBuilderPromptBuilder(
                content_builder=message_builder
            ),
            prompt_processor=SerialProcessor([MarkdownCleaner(), EmojiCleaner()])
        )
        builder = PostBuilder(
            message_builder=message_builder,
            media_builder=image_builder
        )
        super().__init__(builder, posters)  
    
class ProjectBlogger(SimplestBlogger):
    def __init__(self, posters, index=None):
        self.index = index
        super().__init__(self._builder(), posters)

    def _message_prompt_constructor(self, task):
        return f"Напиши пост на тему {task['topic']} из области '{task['category']}', используй не более 100 слов, используй смайлики"
    
    def root_folder(self):
        return './files'
    
    def ideas_folder(self):
        return os.path.join(self.root_folder(), 'ideas')
    
    def projects_folder(self):
        return os.path.join(self.root_folder(), 'projects')
    
    def _tasks_file_path(self):
        return f"{self.root_folder()}/projects/in_progress{(self.index or '')}.json"
    
    def _load_tasks(self):
        return json.load(open(self._tasks_file_path(), "rt", encoding="UTF-8"))
    
    def _check_task(self, task, days_before=0, **_):
        check_date = date.today() + timedelta(days=days_before)
        return task['date'] == check_date.strftime('%Y-%m-%d')
    
    def _task_extractor(self, tasks):
        return TaskExtractor(tasks=tasks, check=self._check_task)           
    
    def print_current_task(self):
        tasks = self._load_tasks()
        task_extractor = self._task_extractor(tasks)
        task = task_extractor.build()
        print(f"system: {self._system_prompt()}")
        print(f"message: {self._message_prompt_constructor(task)}")
        return task
    
    @abstractmethod
    def _builder(self):
        """Build Post Builder"""

    def init_project(self, create_idea_example=False):
        os.makedirs(self.root_folder(), exist_ok=True)
        os.makedirs(self.ideas_folder(), exist_ok=True)
        os.makedirs(self.projects_folder(), exist_ok=True)
        if create_idea_example:
            ideas_example_file = os.path.join(self.ideas_folder(), 'idea.json')
            if not os.path.exists(ideas_example_file):
                json.dump({ 'topic': 'topic', 'category': 'category' }, open(ideas_example_file, 'wt', encoding='UTF-8'))

    def _load_project_tasks(self, multiple_projects=False):
        project_tasks=[]
        tasks=[]
        for root, _, files in os.walk(self.ideas_folder(), ):
            for i, file in enumerate(files):
                input_file = f"{root}/{file}"
                data = json.load(open(input_file, "rt", encoding="UTF-8"))
                for idea in data:
                    task = idea
                    task['index'] = i
                    tasks.append(task)
                if multiple_projects:
                    project_tasks.append(tasks)
                    tasks=[]
        if not multiple_projects:
            project_tasks.append(tasks)
        return project_tasks
    
    def _save_tasks(self, project_tasks):
        for i, tasks in enumerate(project_tasks):
            tasks_file=os.path.join(self.projects_folder(), f"in_progress{'' if i==0 else i}.json")
            json.dump(tasks, open(tasks_file, 'wt', encoding='UTF-8'), indent=4, ensure_ascii=False)

    def _shuffle(self, project_tasks):
        year = date.today().year
        for tasks in project_tasks:
            random.seed(year)
            random.shuffle(tasks)

class CachedBlogger(ProjectBlogger):
    def __init__(self, force_rebuild=False):
        self.force_rebuild = force_rebuild
    
    def _path_constructor(self, task):
        return os.path.join(task['category'], task['topic'])
    
    def _data_folder(self):
        return os.path.join(self.root_folder(), "data")
    
    def print_current_task(self):
        task = super().print_current_task()
        print(f"path: {self._path_constructor(task)}")
        return task
    
    def init_project(self, create_idea_example=False):
        super().init_project(create_idea_example)
        os.makedirs(self._data_folder(), exist_ok=True)




from simple_blogger.uploader.S3Uploader import S3Uploader
from simple_blogger.preprocessor.text import SerialProcessor, MarkdownCleaner, IdentityProcessor
from simple_blogger.poster import IPoster, Post
import os, requests

class InstagramPoster(IPoster):
    def __init__(self, account_token_name='IG_BOT_TOKEN', account_id=None, uploader=S3Uploader(), processor=None, **_):
        self.uploader = uploader
        self.account_token = os.environ.get(account_token_name)
        self.account_id = account_id or self.me()['id']
        self.processor = SerialProcessor([MarkdownCleaner(), processor or IdentityProcessor()])
            
    def post(self, post:Post, processor=None, **_):
        if post.media and post.message:
            media_url = self.uploader.upload(file=post.media)
            caption = post.get_real_message(SerialProcessor([self.processor, processor or IdentityProcessor()]))
            post = self.create_post(self.account_id, media_url=media_url, caption=caption)
            self.publish(self.account_id, post['id'])

    def me(self):
        payload = { 'fields': ['user_id', 'username'], 'access_token': self.account_token }
        user_url = "https://graph.instagram.com/v22.0/me"
        response = requests.get(user_url, params=payload).json()
        return response

    def create_post(self, account_id, media_url, caption):
        payload = { 'image_url': media_url, 'access_token': self.account_token, 'caption': caption }
        crate_image_url = f"https://graph.instagram.com/v22.0/{account_id}/media"
        response = requests.post(crate_image_url, params=payload).json()
        return response

    def publish(self, account_id, creation_id):
        payload = { 'creation_id': creation_id, 'access_token': self.account_token }
        crate_image_url = f"https://graph.instagram.com/v22.0/{account_id}/media_publish"
        response = requests.post(crate_image_url, params=payload).json()
        return response

# class InstagramVideoPoster(InstagramPoster):
#     def create_post(self, account_id, media_url, caption):
#         payload = { 'video_url': media_url, 'media_type': 'VIDEO', 'access_token': self.account_token, 'caption': caption }
#         crate_image_url = f"https://graph.instagram.com/v22.0/{account_id}/media"
#         response = requests.post(crate_image_url, params=payload).json()
#         return response
    
class InstagramReelsPoster(InstagramPoster):
    def create_post(self, account_id, media_url, caption):
        payload = { 'video_url': media_url, 'media_type': 'REELS', 'access_token': self.account_token, 'caption': caption }
        crate_image_url = f"https://graph.instagram.com/v22.0/{account_id}/media"
        response = requests.post(crate_image_url, params=payload).json()
        return response


from simple_blogger.generator import File
from simple_blogger.generator.video import VideoGenerator
from simple_blogger.generator import ImageGenerator
from moviepy import ImageSequenceClip
import tempfile, uuid, io

class ImageSequenceGenerator(VideoGenerator):
    def __init__(self, image_generator:ImageGenerator, duration_constructor=lambda:10, image_count=5):
        self.image_generator=image_generator
        self.duration_constructor=duration_constructor
        self.image_count=image_count

    def _savefile(self, file:File)->str:
        temp_file_name = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{file.ext}"
        open(temp_file_name, 'wb').write(file.get_file().read())
        return temp_file_name

    def generate(self, prompt=None):
        images = [self.image_generator.generate(prompt) for _ in range(self.image_count)]
        duration = self.duration_constructor()
        image_duration = duration / self.image_count
        clip = ImageSequenceClip(
            sequence=[self._savefile(image) for image in images],
            durations=[image_duration]*self.image_count)
        temp_file_name = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.ext()}"
        clip.write_videofile(temp_file_name, fps=self.fps())
        return File(self.ext(), io.BytesIO(open(temp_file_name,'rb').read()))


from simple_blogger.generator import File, ImageGenerator, TextGenerator
from gigachat import GigaChat
from io import StringIO, BytesIO
from bs4 import BeautifulSoup
import os, base64

class GigaChatTextGenerator(TextGenerator):
    def __init__(self, system_prompt, api_key_name ='GIGACHAT_CREDENTIALS', model_name='GigaChat-2', creativity=0.5):
        super().__init__(system_prompt=system_prompt)
        self.api_key = os.environ.get(api_key_name)
        self.model_name=model_name
        self.creativity=creativity*2

    def generate(self, prompt, **_):
        giga = GigaChat(
            credentials=self.api_key,
            verify_ssl_certs=False,
        )
        text = giga.chat({
                                "model": f"{self.model_name}",
                                "messages": [
                                    { "role": "system", "content": f"{self.system_prompt}" },
                                    { "role": "user", "content": f"{prompt}" },
                                ],
                                "temperature": self.creativity
                            }).choices[0].message.content
        return File('txt', StringIO(text))
    
class GigaChatImageGenerator(ImageGenerator):
    def __init__(self, api_key_name ='GIGACHAT_CREDENTIALS', model_name='GigaChat-2', system_prompt=None, style_prompt=None):
        super().__init__()
        self.api_key = os.environ.get(api_key_name)
        self.model_name=model_name
        self.system_prompt=system_prompt
        self.style_prompt=style_prompt

    def generate(self, prompt, **_):
        giga = GigaChat(
            credentials=self.api_key,
            verify_ssl_certs=False,
        )
        prompt = f"{prompt}{'\n' + self.style_prompt if self.style_prompt else ''}"
        response = giga.chat({
                                "model": f"{self.model_name}",
                                "messages": [
                                    { "role": "system", "content": f"{self.system_prompt}" },
                                    { "role": "user", "content": f"{prompt}" },
                                ],
                                "function_call": "auto"
                            }).choices[0].message.content
        
        file_id = BeautifulSoup(response, "html.parser").find('img').get("src")
        image = giga.get_image(file_id)

        return File(self.ext(), BytesIO(base64.b64decode(image.content)))
    
    def ext(self):
        return 'jpeg'


from moviepy.audio.io.AudioFileClip import AudioFileClip 
from moviepy.video.tools.subtitles import SubtitlesClip
from moviepy.video.io.VideoFileClip import VideoFileClip
from moviepy import CompositeVideoClip
import tempfile, uuid, os
from io import BytesIO
from simple_blogger.generator import File
from simple_blogger.builder.content import IContentBuilder
import simple_blogger.resources

class ShortsBuilder(IContentBuilder):
    def __init__(self, 
                 audio_builder:IContentBuilder,
                 subs_builder:IContentBuilder,
                 video_builder:IContentBuilder,
                 font=None
                 ):
        self.audio_builder=audio_builder
        self.subs_builder=subs_builder
        self.video_builder=video_builder
        self.font=font or os.path.join(*simple_blogger.resources.__path__, 'Mulish', 'static', 'Mulish-Bold.ttf')

    def ext(self):
        return self.video_builder.ext()

    def build(self):
        audio_filename = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.audio_builder.ext()}"
        open(audio_filename, 'wb').write(self.audio_builder.build().get_file().read())
        audio = AudioFileClip(audio_filename)
        
        subs_filename = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.subs_builder.ext()}"
        open(subs_filename, 'wt', encoding='UTF-8').write(self.subs_builder.build().get_file().read())
        subs = SubtitlesClip(subtitles=subs_filename, font=self.font, encoding='UTF-8')
        
        video_filename = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.video_builder.ext()}"
        open(video_filename, 'wb').write(self.video_builder.build().get_file().read())
        video = VideoFileClip(video_filename)
        
        media_filename = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.video_builder.ext()}"
        media = CompositeVideoClip([video, subs.with_position(("center", 0.9), relative=True)]).with_audio(audio)
        media.write_videofile(media_filename)

        return File(self.video_builder.ext(), BytesIO(open(media_filename, 'rb').read()))


from simple_blogger.generator import File, IGeneratorBase
from abc import abstractmethod
from io import StringIO
import srt
from datetime import timedelta

class ISubsGeneratorBase(IGeneratorBase):    
    @abstractmethod
    def generate(self, text_to_sub)->File:
        """Generation method"""

class SubsGenerator(ISubsGeneratorBase):
    def __init__(self, duration_constructor=lambda:10):
        self.duration_constructor = duration_constructor
        pass

    def generate(self, text_to_sub):  
        subs = []
        delta = 3
        speed = len(text_to_sub) / (duration - delta + 1 if (duration:=self.duration_constructor()) > delta else 1)
        text = ' '
        tik = 0
        index = 0
        for word in text_to_sub.split():
            if len(text) + len(word) > speed * delta:
                subs.append(srt.Subtitle(index=index,start=timedelta(seconds=tik), end=timedelta(seconds=tik+delta),content=text))
                text = word
                tik += delta
                index += 1
            else:
                text += f" {word}"
        if text != '':
            subs.append(srt.Subtitle(index=index,start=timedelta(seconds=tik), end=timedelta(seconds=duration)-timedelta(milliseconds=100) ,content=text))
        stream = StringIO(srt.compose(subtitles=subs, reindex=False))
        return File(self.ext(), stream)
    
    def ext(self):
        return 'srt'


from setuptools import setup, find_packages

setup(
  name='simple_blogger',
  version='0.2.13',
  author='Aleksey Sergeyev',
  author_email='aleksey.sergeyev@yandex.com',
  description='A simple blogger library',
  long_description=open('README.md', 'rt').read(),
  long_description_content_type='text/markdown',
  url='https://github.com/athenova/simple_blogger',
  packages=find_packages(),
  install_requires=['pillow>=10.4.0', 
                    'openai>=1.76.0', 
                    'pyTelegramBotAPI>=4.26.0', 
                    'requests>=2.32.3', 
                    'yandex-cloud-ml-sdk>=0.4.1', 
                    "emoji>=2.14.1",
                    'markdown>=3.7', 
                    "vk>=3.0",
                    "gigachat>=0.1.42",
                    "beautifulsoup4>=4.13.4",
                    # 'boto3==1.35.99', 
                    # "yandex-speechkit>=1.5.0",
                    # "moviepy>=2.1.2"
                    ],
  classifiers=[
    'Programming Language :: Python :: 3.10',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
  ],
  keywords='python blog ai',
  project_urls={
    'Documentation': 'https://github.com/athenova/simple_blogger'
  },
  python_requires='>=3.10.6'
)


from simple_blogger.blogger.shorts import ShortsBlogger
from simple_blogger.blogger.shorts.cached import CachedShortsBlogger
from simple_blogger.blogger.finite import FiniteBlogger

class FiniteCommonShortsBlogger(ShortsBlogger, FiniteBlogger):
    def __init__(self, posters, image_count=5, index=None):
        ShortsBlogger.__init__(self, posters, image_count, index)

class CacheFiniteShortBlogger(CachedShortsBlogger, FiniteBlogger):
    def __init__(self, posters, image_count=5, force_rebuild=False, index=None):
        CachedShortsBlogger.__init__(self, posters, image_count, force_rebuild, index)


from speechkit import configure_credentials, creds
from speechkit import model_repository
from simple_blogger.generator.speech import SpeechGenerator, File
import os
from io import BytesIO

configure_credentials(
    yandex_credentials=creds.YandexCredentials(
        api_key=os.environ.get('YC_API_KEY')
    )
)

class YandexSpeechGenerator(SpeechGenerator):
    def __init__(self, voice='masha', role='good', speed= 1.2):
        self.voice=voice
        self.role=role
        self.speed=speed

    def generate(self, text_to_speak):
        model = model_repository.synthesis_model()
        model.voice = self.voice
        model.role = self.role
        model.speed = self.speed
        model.unsafe_mode = True   
        result = model.synthesize(text_to_speak, raw_format=False)
        stream=BytesIO()
        result.export(stream, format=self.ext())
        return File(self.ext(), stream)


from simple_blogger.generator import File, TextGenerator, ImageGenerator
from openai import OpenAI
from io import StringIO, BytesIO
import os, requests, base64

class OpenAiTextGenerator(TextGenerator):
    def __init__(self, system_prompt, api_key_name ='OPENAI_API_KEY', model_name='chatgpt-4o-latest', creativity=0.5):
        super().__init__(system_prompt=system_prompt)
        self.api_key = os.environ.get(api_key_name)
        self.model_name=model_name
        self.creativity=creativity*2

    def generate(self, prompt, **_):
        client = OpenAI(api_key=self.api_key)
        text = client.chat.completions.create(
                    model=self.model_name,
                    messages=[
                        { "role": "system", "content": self.system_prompt },
                        { "role": "user", "content": prompt },
                    ],
                    temperature=self.creativity
                ).choices[0].message.content
        return File(self.ext(), StringIO(text))
    
class OpenAiImageGenerator(ImageGenerator):
    def __init__(self, api_key_name ='OPENAI_API_KEY', model_name='dall-e-3', system_prompt=None, style_prompt=None, size="1024x1024", quality="standard"):
        self.api_key = os.environ.get(api_key_name)
        self.model_name=model_name
        self.system_prompt=system_prompt
        self.style_prompt=style_prompt
        self.size=size
        self.quality=quality

    def generate(self, prompt, **_):
        client = OpenAI(api_key=self.api_key)
        prompt = f"{self.system_prompt + '\n' if self.system_prompt else ''}{prompt}{'\n' + self.style_prompt if self.style_prompt else ''}"
        image_url = client.images.generate(
            model = self.model_name,
            prompt = prompt,
            size = self.size,
            quality = self.quality,
            n = 1                
        ).data[0].url
        response = requests.get(image_url)
        return File(self.ext(), BytesIO(response.content))
    
    def ext(self):
        return "png"
    
class GptImageGenerator(ImageGenerator):
    def __init__(self, api_key_name ='OPENAI_API_KEY', model_name='gpt-image-1', system_prompt=None, style_prompt=None, size="1024x1024", quality="medium", output_format="png"):
        self.api_key = os.environ.get(api_key_name)
        self.model_name=model_name
        self.system_prompt=system_prompt
        self.style_prompt=style_prompt
        self.size=size
        self.quality=quality
        self.output_format=output_format

    def generate(self, prompt, **_):
        client = OpenAI(api_key=self.api_key)
        prompt = f"{self.system_prompt + '\n' if self.system_prompt else ''}{prompt}{'\n' + self.style_prompt if self.style_prompt else ''}"
        image_base64 = client.images.generate(
            model = self.model_name,
            prompt = prompt,
            size = self.size,
            quality = self.quality
        ).data[0].b64_json
        image_bytes = base64.b64decode(image_base64)
        return File(self.ext(), BytesIO(image_bytes))
    
    def ext(self):
        return self.output_format


from simple_blogger.generator import File, TextGenerator
from openai import OpenAI
from io import StringIO
import os

class DeepSeekTextGenerator(TextGenerator):
    def __init__(self, system_prompt, api_key_name ='DEEPSEEK_API_KEY', model_name='deepseek-chat', creativity=0.5):
        super().__init__(system_prompt=system_prompt)
        self.base_url='https://api.deepseek.com'
        self.api_key = os.environ.get(api_key_name)
        self.model_name=model_name
        self.creativity=creativity*2

    def generate(self, prompt, **_):
        client = OpenAI(api_key=self.api_key, base_url=self.base_url)
        text = client.chat.completions.create(
                    model=self.model_name,
                    messages=[
                        { "role": "system", "content": self.system_prompt },
                        { "role": "user", "content": prompt },
                    ],
                    temperature=self.creativity
                ).choices[0].message.content
        return File('txt', StringIO(text))


from simple_blogger.builder.content import IContentBuilder
from moviepy.audio.io.AudioFileClip import AudioFileClip 
import tempfile, uuid, os

class DurationBuilder():
    def __init__(self, content_builder:IContentBuilder):
        self.content_builder=content_builder
    
    def build(self)->float:
        audio_filename = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.content_builder.ext()}"
        open(audio_filename, 'wb').write(self.content_builder.build().get_file().read())
        audio = AudioFileClip(audio_filename)
        return audio.duration


from simple_blogger.poster import Post

class PostBuilder():
    def __init__(self, message_builder=None, media_builder=None):
        self.message_builder = message_builder
        self.media_builder = media_builder

    def build(self):
        return Post(
            self.message_builder and self.message_builder.build(),
            self.media_builder and self.media_builder.build()
        )


from simple_blogger.generator import IGeneratorBase, File, ImageGenerator
import tempfile, uuid
from moviepy import ImageClip

class VideoGenerator(IGeneratorBase):
    def __init__(self):
        pass

    def generate(self, _=None):       
        image = ImageGenerator().generate()
        clip = ImageClip(image.get_file(), duration=3)
        temp_file = f"{tempfile.gettempdir()}/{str(uuid.uuid4())}.{self.ext()}"
        clip.write_videofile(temp_file, fps=self.fps())
        return File(self.ext(), open(temp_file,'rb'))
    
    def fps(self):
        return 24
    
    def ext(self):
        return 'mp4'


from abc import ABC, abstractmethod

class ITaskBuilder(ABC):
    @abstractmethod
    def build(self):
        """Task extractor method """

class IdentityTaskBuilder(ITaskBuilder):
    def __init__(self, task):
        self.task = task

    def build(self):
        return self.task

class TaskExtractor(ITaskBuilder):
    def __init__(self, tasks, check):
        self.tasks = tasks
        self.check = check

    def build(self):
        for task in self.tasks:
            if self.check(task=task, tasks=self.tasks):
                return task
        return None


from __future__ import annotations 
import simple_blogger.builder.prompt, os
from simple_blogger.generator import File
from simple_blogger.cache.file_system import FileCache
from simple_blogger.builder.task import ITaskBuilder
from simple_blogger.preprocessor.text import ITextProcessor, IdentityProcessor
from abc import ABC, abstractmethod

class IContentBuilder(ABC):    
    @abstractmethod
    def build(self)->File:
        """Content builder method"""

    @abstractmethod
    def ext(self)->str:
        """Content extension"""

class ContentBuilder(IContentBuilder):
    def __init__(self, generator, prompt_builder:simple_blogger.builder.prompt.IPromptBuilder, prompt_processor:ITextProcessor=None):
        self.generator = generator
        self.prompt_builder = prompt_builder
        self.prompt_processor=prompt_processor or IdentityProcessor()

    def build(self):
        prompt = self.prompt_processor.process(self.prompt_builder.build())
        return prompt and self.generator.generate(prompt)
    
    def ext(self):
        return self.generator.ext()

class CachedContentBuilder(IContentBuilder):
    def __init__(self, task_builder:ITaskBuilder, path_constructor, builder:IContentBuilder, force_rebuild=False, cache=None, filename='topic'):
        self.task_builder = task_builder
        self.path_constructor = path_constructor
        self.builder = builder
        self.force_rebuild = force_rebuild
        self.cache = cache or FileCache(is_binary = builder.ext() != 'txt')
        self.filename = filename

    def __build(self, task):
        uri = os.path.join(self.path_constructor(task), f"{self.filename}.{self.builder.ext()}")
        if self.force_rebuild or (cached := self.cache.load(uri=uri)) is None:
            new = self.builder.build()
            if new is not None:
                self.cache.save(uri=uri, io_base=new.file)
            return new
        return File(self.builder.ext(), cached) 

    def build(self, **__):
        task = self.task_builder.build()
        return task and self.__build(task)
    
    def ext(self):
        return self.builder.ext()